import { Selector, t } from 'testcafe';

export const FileMenu = {
  Menu: Selector('#file'),
  New: Selector('#file_new'),
  Clear: Selector('#file_clear'),
  Open: Selector('#file_open'),
  Save: Selector('#file_save'),
  SaveAs: Selector('#file_saveAs'),
  SaveAsStart: Selector('#file_saveAsStart'),
  Export: Selector('#file_export'),
  Exit: Selector('#file_exit'),
};

export const ToolsMenu = {
  Menu: Selector('#tools'),
  ManageConnections: Selector('#tools_manageConnections'),
  ResetServiceStates: Selector('#tools_resetServiceStates'),
  ResetLayout: Selector('#tools_resetLayout'),
  ResetDriver: Selector('#tools_resetDriver'),
  ScanDevices: Selector('#tools_scanDevices'),
  SetLanguage: Selector('#tools_setLanguage'),
  SetLanguage_DE: Selector('#tools_setLanguage_DE'),
  SetLanguage_EN: Selector('#tools_setLanguage_EN'),
  SetLanguage_ES: Selector('#tools_setLanguage_ES'),
  SetLanguage_FR: Selector('#tools_setLanguage_FR'),
  SetLanguage_IT: Selector('#tools_setLanguage_IT'),
  SetLanguage_PT: Selector('#tools_setLanguage_PT'),
  UserSettings: Selector('#tools_userSettings'),
};

type TMenu = {
  Menu: Selector;
  [index: string]: Selector;
};

export async function clickMenu<T extends TMenu>(
  t: TestController,
  x: T,
  key: keyof Omit<T, 'Menu'>,
): Promise<void> {
  return t.hover(x.Menu).click(x[key]);
}

const DeviceSelector = Selector('.deviceImage');

// TODO: Generate from current catalog??
export const DeviceIds = {
  Core3: 'RevPiCore_20170404_1_2',
  Connect4: 'RevPiConnect4_20230409_1_0',
  DIO: 'RevPiDIO_20160818_1_0',
  MIO: 'RevPiMIO_20200901_1_0',
  RO: 'RevPiRO_20231018_1_0',
  Gateway_DMX: 'RevPiGateDMX_20160824_1_0',
};

export const Editor = {
  DropTarget: Selector('#dragTable td.dropCell'),
  DropTargetEdgeLeft: Selector('#dragTable td.dropCell.edge').nth(0),
  DropTargetEdgeRight: Selector('#dragTable td.dropCell.edge').nth(1),
  AllDevices: DeviceSelector,
  BaseDevice: DeviceSelector.filter('.isBase'),
  LeftDevice: DeviceSelector.filter('.isLeft'),
  RightDevice: DeviceSelector.filter('.isRight'),

  CurrentDevices: {
    Core3: Selector(`#device_${DeviceIds.Core3}_001`),
    Connect4: Selector(`#device_${DeviceIds.Connect4}_001`),
    DIO: Selector(`#device_${DeviceIds.DIO}_001`),
    MIO: Selector(`#device_${DeviceIds.MIO}_001`),
    RO: Selector(`#device_${DeviceIds.RO}_001`),
    Gateway_DMX: Selector(`#device_${DeviceIds.Gateway_DMX}_001`),
  },
};

export const ValueEditor = {
  Table: Selector('#tblEdit'),
  getRow: (x: number) => ValueEditor.Table.find('tbody tr').nth(x),
  getColumnsOfRow: (x: number) => {
    const cols = ValueEditor.getRow(x).find('td');

    return {
      Type: cols.nth(0),
      Name: cols.nth(1),
      Value: cols.nth(2),
      Unit: cols.nth(3),
      Comment: cols.nth(4),
      Export: cols.nth(5),
    };
  },
};

export const DeviceCatalog = {
  CatalogTree: Selector('#catalogTree'),
  Base: {
    Folder: Selector('#ft_f_01'),
    Core3: Selector(`#ft_${DeviceIds.Core3}`),
    Connect4: Selector(`#ft_${DeviceIds.Connect4}`),
  },
  Gateways: {
    Folder: Selector('#ft_f_03'),
    Gateway_DMX: Selector(`#ft_${DeviceIds.Gateway_DMX}`),
  },
  IO: {
    Folder: Selector('#ft_f_02'),
    DIO: Selector(`#ft_${DeviceIds.DIO}`),
    MIO: Selector(`#ft_${DeviceIds.MIO}`),
    RO: Selector(`#ft_${DeviceIds.RO}`),
  },
  VirtualDevices: {
    Folder: Selector('#ft_f_04'),
  },
  VirtualModBusDevices: {
    Folder: Selector('#ft_f_05'),
  },
};

export async function AddCore3(): Promise<void> {
  await t
    .dragToElement(DeviceCatalog.Base.Core3, Editor.DropTarget)
    .expect(Editor.CurrentDevices.Core3.exists)
    .ok()
    .expect(Editor.BaseDevice.id)
    .contains(DeviceIds.Core3);
}

export async function AddConnect4(): Promise<void> {
  await t
    .dragToElement(DeviceCatalog.Base.Connect4, Editor.DropTarget)
    .expect(Editor.CurrentDevices.Connect4.exists)
    .ok()
    .expect(Editor.BaseDevice.id)
    .contains(DeviceIds.Connect4);
}

export async function AddDIO(): Promise<void> {
  await t
    .dragToElement(DeviceCatalog.IO.DIO, Editor.DropTargetEdgeLeft)
    .expect(Editor.CurrentDevices.DIO.exists)
    .ok()
    .expect(Editor.LeftDevice.id)
    .contains(DeviceIds.DIO);
}

export async function AddMIO(): Promise<void> {
  await t
    .dragToElement(DeviceCatalog.IO.MIO, Editor.DropTargetEdgeRight)
    .expect(Editor.CurrentDevices.MIO.exists)
    .ok()
    .expect(Editor.RightDevice.id)
    .contains(DeviceIds.MIO);
}

export async function AddRO(): Promise<void> {
  await t
    .dragToElement(DeviceCatalog.IO.RO, Editor.DropTargetEdgeRight)
    .expect(Editor.CurrentDevices.RO.exists)
    .ok()
    .expect(Editor.RightDevice.id)
    .contains(DeviceIds.RO);
}
